#include "H264AVCCommonLib.h"
#include "H264AVCCommonLib/DepthRange.h"

//#if SONY_DEPTH_RANGE

/*
AcquisitionElement::AcquisitionElement() :
  m_ucSkipFlag( NULL )
, m_ucElementEqual( 0 )
{
}
*/

H264AVC_NAMESPACE_BEGIN

DepthRange::DepthRange():
m_uiMaxZ	( NULL ),
m_uiMinZ	( NULL ),
m_ucMaxZElementEqualFlag ( NULL ),
m_ucMinZElementEqualFlag ( NULL ),
m_ucMaxZPresentFlag ( NULL ),
m_ucMinZPresentFlag ( NULL ),
m_uiAllDepthRangeSameFlag	(0),
m_uiNumFramesMinus1 (0),
m_uiNumDepthViewsMinus1 (0)
{
}

ErrVal DepthRange::init(UInt uiAllDepthRangeSameFlag,
						UInt uiNumDepthViewsMinus1, 
						UInt uiNumFramesMinus1)
{
	m_uiAllDepthRangeSameFlag = uiAllDepthRangeSameFlag;
	m_uiNumDepthViewsMinus1 = uiNumDepthViewsMinus1;
	m_uiNumFramesMinus1 = uiNumFramesMinus1;

	if(m_uiAllDepthRangeSameFlag){
		m_uiMaxZ = new AcquisitionElement*[1];
		m_uiMaxZ[0] = new AcquisitionElement[1];
		m_ucMaxZElementEqualFlag = new UChar[1];
		m_ucMaxZPresentFlag = new UChar[1];

		m_uiMinZ = new AcquisitionElement*[1];
		m_uiMinZ[0] = new AcquisitionElement[1];
		m_ucMinZElementEqualFlag = new UChar[1];
		m_ucMinZPresentFlag = new UChar[1];
	}else{
		m_uiMaxZ = new AcquisitionElement*[(m_uiNumDepthViewsMinus1+1)];
		for(int i=0; i< (m_uiNumDepthViewsMinus1+1); i++) {
			m_uiMaxZ[i] = new AcquisitionElement[(m_uiNumFramesMinus1+1)];
		}
		m_ucMaxZElementEqualFlag = new UChar[m_uiNumFramesMinus1+1];
		m_ucMaxZPresentFlag = new UChar[m_uiNumFramesMinus1+1];

		m_uiMinZ = new AcquisitionElement*[(m_uiNumDepthViewsMinus1+1)];
		for(int i=0; i< (m_uiNumDepthViewsMinus1+1); i++) {
			m_uiMinZ[i] = new AcquisitionElement[(m_uiNumFramesMinus1+1)];
		}
		m_ucMinZElementEqualFlag = new UChar[m_uiNumFramesMinus1+1];
		m_ucMinZPresentFlag = new UChar[m_uiNumFramesMinus1+1];

	}
	return Err::m_nOK;
}

ErrVal DepthRange::uninit(UInt uiAllDepthRangeSameFlag,
						UInt uiNumDepthViewsMinus1)
{
	delete [] m_uiMaxZ;
	delete [] m_uiMinZ;

	delete [] m_ucMaxZElementEqualFlag;
	delete [] m_ucMaxZPresentFlag;

    if(m_uiAllDepthRangeSameFlag)
    {
      delete [] m_uiMaxZ[0];
      delete [] m_uiMinZ[0];
	}else{
	  for(int i=0; i< (m_uiNumDepthViewsMinus1+1); i++) {
        delete [] m_uiMaxZ[i];
        delete [] m_uiMinZ[i];
	  }
	}

	return Err::m_nOK;
}

Int32 AcquisitionElement::getVarLength(Int32 iExponent, Int32 iPrecision) 
{
	if (iExponent==0)
		return max(0, -30+iPrecision);
	else 
		return max(0, iExponent-31+iPrecision);
}

void AcquisitionElement::CalcExponentMantissa ()
{
	Double dExponent;
	Double dMantissa;
	Int32  iExponent;
	Int32  iMantissa=0;
	Int32  iTotalBit;
	Int32  iBits;
	

	if(m_ucPrecMode){
		if((Double)m_iOrgValue==0.0){
			iExponent = 0;
			dMantissa = 0.0;
		}else{
			dExponent = log((Double)m_iOrgValue)/log(2.0);
			iExponent = (Int32)floor(dExponent);
			dMantissa = pow(2.0, dExponent - m_iExponent);
			if(fabs(dMantissa) > 1.0){
				dMantissa = dMantissa / 2.0;
				iExponent ++;
			}
		}

		if(dExponent == 0 && dMantissa == 0.0){
			iTotalBit = 0;
		}else if( iExponent > -30){
			while (fabs(dMantissa) < 1.0){
				dMantissa *= 2.0;
				iExponent--;
			}
			if(dMantissa >= 0.0) dMantissa = dMantissa - 1.0;
			iTotalBit = iExponent + m_iPrecision;
			if(iTotalBit < 0) iTotalBit = 0;

			iExponent += 31;
		}else if(iExponent == -31){
			iTotalBit = iExponent + m_iPrecision;
			if(iTotalBit < 0) iTotalBit = 0;
			
			iExponent = 0;
		}

		if( iTotalBit = 0 > 30) iTotalBit = 30;

		for(int i=1;i <= iTotalBit; i++){
			iBits = (2.0 * dMantissa >= 1.0);
			dMantissa = 2.0 * dMantissa - (Double)iBits;
			iMantissa = (iMantissa << 1) | iBits;
		}
	}else{
		dExponent = log((Double)m_iOrgValue)/log(2.0);
		iExponent = (Int32)floor(dExponent);
		
		iExponent += 31;

		iMantissa = floor((((Double)m_iOrgValue)/pow(2.0,dExponent)-1)*pow(2.0,(Int32)m_uiMantissaLength)+0.5);
		iTotalBit = m_uiMantissaLength;

		m_iExponent = iExponent;
		m_iMantissa = iMantissa;
		m_iMantissaBits = iTotalBit;
		m_iSign = (m_iOrgValue < 0 ? -1 : 1);
	}

}

void AcquisitionElement::CalcRecValue ()
{
	Int32 iLength;
	Double dFactor = 1.0;
	if(m_ucPrecMode){
		iLength = getVarLength(m_iExponent, m_iPrecision);
		if(iLength > 30) iLength = 30;
		for(int i=0; i<iLength; i++) dFactor /= 2.0;

		for(int i=0; i<iLength; i++){
			m_iRecValue = (Int32) dFactor * ((m_iMantissa >> i)&0x01);
			dFactor *= 2.0;
		}
		if(m_iExponent > 0 && m_iExponent < m_iExponentSize)
			m_iRecValue = (Int32) m_iSign * (Double)pow(2.0, m_iExponent-31) * (1.0+m_iRecValue);
		if(m_iExponent == 0)
			m_iRecValue = (Int32) m_iSign * (Double)pow(2.0, -30.0) * m_iRecValue;
	}else{
		m_iRecValue = (Double) m_iSign * pow(2.0,m_iExponent-31) * (1+ m_iMantissa / pow(2.0, (Int32)m_uiMantissaLength));
	}
}

ErrVal
AcquisitionElement::write( HeaderSymbolWriteIf* pcWriteIf) const
{

    //if(m_ucPrecMode == 2 && i == 0)
	  if(m_ucPrecMode == 0){
		RNOK  ( pcWriteIf->writeCode(  m_uiMantissaLength - 1,             5,      "ACQUISITION ELEMENT: mantissa_len_minus1" ) );
	  }else{
		RNOK  ( pcWriteIf->writeCode(  m_iPrecision,                      5,      "ACQUISITION ELEMENT: prec" ) );
	  }

	  //if(m_ucPrecMode == 2)
	  RNOK  ( pcWriteIf->writeFlag( m_iSign,                                      "ACQUISITION ELEMENT: sign0" ) );
	  RNOK  ( pcWriteIf->writeCode( m_iExponent,                m_iExponentSize,  "ACQUISITION ELEMENT: exponent0" ) );
	  //if(m_uiPrecMode == 1){
	  //  if(m_iExponent == 0)
	  RNOK  ( pcWriteIf->writeCode( m_iMantissa,                m_uiMantissaLength,  "ACQUISITION ELEMENT: mantissa0" ) );

	  return Err::m_nOK;
}



ErrVal DepthRange::create( DepthRange*& rpcDepthRange )
{
	rpcDepthRange = new DepthRange;
	ROT( NULL == rpcDepthRange );
	return Err::m_nOK;
}

ErrVal
DepthRange::write( HeaderSymbolWriteIf* pcWriteIf, 
				   UInt uiNumViewsMinus1,
				   UInt uiFrameNum,
				   Bool b3dvExtensionSPSFlag) const
{

  UInt uiNumViews;

  if (!b3dvExtensionSPSFlag){
	  //===== NAL unit header =====
	  //ETRACE_DECLARE( Bool m_bTraceEnable = true );
	  //g_nLayer = m_uiLayerId;
	  //ETRACE_LAYER(m_uiLayerId);
	  //ETRACE_VIEWID(m_uiCurrentViewId);
	  //ETRACE_HEADER( "SEQUENCE PARAMETER SET" );
	  RNOK  ( pcWriteIf->writeFlag( 0,                                        "NALU HEADER: forbidden_zero_bit" ) );
	  RNOK  ( pcWriteIf->writeCode( 0, 2,                                     "NALU HEADER: nal_ref_idc" ) );
	  RNOK  ( pcWriteIf->writeCode( NAL_UNIT_DEPTH_RANGE_UPDATE, 5,           "NALU HEADER: nal_unit_type" ) );
  }

  //===== depth ranges =====
	RNOK  ( pcWriteIf->writeFlag( m_ucMinZPresentFlag[uiFrameNum],             "DEPTH RANGE: z_near_flag" ) );
	if(m_ucMinZPresentFlag[uiFrameNum]){
		if (uiNumViewsMinus1 + 1 > 1){
		  RNOK  ( pcWriteIf->writeFlag( m_ucMinZElementEqualFlag[uiFrameNum],  "ACQUISITION ELEMENT: element_equal_flag" ) );
	    }
	    if(m_ucMinZElementEqualFlag[uiFrameNum] == 0){
		  uiNumViews = uiNumViewsMinus1 + 1;
	    }else{
		  uiNumViews = 1;
	    }
	    for(int i=0; i < uiNumViews; i++){
		  m_uiMinZ[i][uiFrameNum].write(pcWriteIf);
		}
	}

	RNOK  ( pcWriteIf->writeFlag( m_ucMaxZPresentFlag[uiFrameNum],            "DEPTH RANGE: z_far_flag" ) );
	if(m_ucMaxZPresentFlag[uiFrameNum]){
		if (uiNumViewsMinus1 + 1 > 1){
		  RNOK  ( pcWriteIf->writeFlag( m_ucMaxZElementEqualFlag[uiFrameNum], "ACQUISITION ELEMENT: element_equal_flag" ) );
	    }
	    if(m_ucMaxZElementEqualFlag[uiFrameNum] == 0){
		  uiNumViews = uiNumViewsMinus1 + 1;
	    }else{
		  uiNumViews = 1;
	    }
	    for(int i=0; i < uiNumViews; i++){
		  m_uiMaxZ[i][uiFrameNum].write(pcWriteIf);
		}
	}

  return Err::m_nOK;
}


	/*
DepthRange& DepthRange::operator = ( const DepthRange& rcDepthRange )
{

	m_uiMaxZ = rcDepthRange.m_uiMaxZ;
	m_uiMinZ = rcDepthRange.m_uiMinZ;
    m_uiAllDepthRangeSameFlag = rcDepthRange.m_uiAllDepthRangeSameFlag;
    m_uiNumDepthInfoMinus1 = rcDepthRange.m_uiNumDepthInfoMinus1;
    m_uiNumFramesMinus1 = rcDepthRange.m_uiNumFramesMinus1;
    m_uiNumDepthViewsMinus1 = rcDepthRange.m_uiNumDepthViewsMinus1;


    m_eNalUnitType                      = rcSPS.m_eNalUnitType;
    m_uiLayerId                         = rcSPS.m_uiLayerId;
    m_eProfileIdc                       = rcSPS.m_eProfileIdc;
    m_bConstrainedSet0Flag              = rcSPS.m_bConstrainedSet0Flag;
    m_bConstrainedSet1Flag              = rcSPS.m_bConstrainedSet1Flag;
    m_bConstrainedSet2Flag              = rcSPS.m_bConstrainedSet2Flag;
    m_bConstrainedSet3Flag              = rcSPS.m_bConstrainedSet3Flag;
    m_uiLevelIdc                        = rcSPS.m_uiLevelIdc;
    m_uiSeqParameterSetId               = rcSPS.m_uiSeqParameterSetId;
    m_uiLog2MaxFrameNum                 = rcSPS.m_uiLog2MaxFrameNum;
    m_uiPicOrderCntType                 = rcSPS.m_uiPicOrderCntType;
    m_uiLog2MaxPicOrderCntLsb           = rcSPS.m_uiLog2MaxPicOrderCntLsb;
    m_bDeltaPicOrderAlwaysZeroFlag      = rcSPS.m_bDeltaPicOrderAlwaysZeroFlag;
    m_iOffsetForNonRefPic               = rcSPS.m_iOffsetForNonRefPic;
    m_iOffsetForTopToBottomField        = rcSPS.m_iOffsetForTopToBottomField;
    m_uiNumRefFramesInPicOrderCntCycle  = rcSPS.m_uiNumRefFramesInPicOrderCntCycle;
//    m_piOffsetForRefFrame               = rcSPS.m_piOffsetForRefFrame;
    m_uiNumRefFrames                    = rcSPS.m_uiNumRefFrames;
//    m_bGapsInFrameNumValueAllowedFlag   = rcSPS.m_bGapsInFrameNumValueAllowedFlag;
    m_uiFrameWidthInMbs                 = rcSPS.m_uiFrameWidthInMbs;
    m_uiFrameHeightInMbs                = rcSPS.m_uiFrameHeightInMbs;
    m_bFrameMbsOnlyFlag                 = rcSPS.m_bFrameMbsOnlyFlag;
    m_bMbAdaptiveFrameFieldFlag         = rcSPS.m_bMbAdaptiveFrameFieldFlag;
    m_bDirect8x8InferenceFlag           = rcSPS.m_bDirect8x8InferenceFlag;
//    m_bNalUnitExtFlag                   = rcSPS.m_bNalUnitExtFlag;
//    m_uiNumSimplePriIdVals              = rcSPS.m_uiNumSimplePriIdVals;
//#if MULTIPLE_LOOP_DECODING
//    m_bAlwaysDecodeBaseLayer            = rcSPS.m_bAlwaysDecodeBaseLayer;
//#endif
    m_uiExtendedSpatialScalability      = rcSPS.m_uiExtendedSpatialScalability;
    m_uiChromaPhaseXPlus1               = rcSPS.m_uiChromaPhaseXPlus1;
    m_uiChromaPhaseYPlus1               = rcSPS.m_uiChromaPhaseYPlus1;
    m_iScaledBaseLeftOffset             = rcSPS.m_iScaledBaseLeftOffset;
    m_iScaledBaseTopOffset              = rcSPS.m_iScaledBaseTopOffset;
    m_iScaledBaseRightOffset            = rcSPS.m_iScaledBaseRightOffset;
    m_iScaledBaseBottomOffset           = rcSPS.m_iScaledBaseBottomOffset;
    m_bInitDone                         = rcSPS.m_bInitDone;

    return *this;
}
*/
/*
void 
DepthRange::initialize_memory(UInt uiNumDepthInfoMinus1, UInt uiNumFramesMinus1) 
{
 m_uiMaxZ = new AcquisitionElement*[uiNumDepthInfoMinus1+1];
 m_uiMinZ = new AcquisitionElement*[uiNumDepthInfoMinus1+1];
 for( int i=0;i<uiNumDepthInfoMinus1+1;i++)
 {
   m_uiMaxZ[i] = new AcquisitionElement[uiNumFramesMinus1+1];
   m_uiMinZ[i] = new AcquisitionElement[uiNumFramesMinus1+1];
 }
}

void 
DepthRange::release_memory()
{
 for( int i=0;i<m_uiNumDepthInfoMinus1+1;i++ )
 {
   delete [] m_uiMaxZ[i];
   delete [] m_uiMinZ[i];
 }
 delete [] m_uiMaxZ;
 delete [] m_uiMinZ;
}
*/

H264AVC_NAMESPACE_END

//#endif

